/**********************************************************
                  Jan. 21, 2002  Shin Satoh (satoh@crl.go.jp)

Main routine of the TRMM PR Latent Heating Algorithm

SOURCE FILE: glhret_v3.1.c  <<< version 3.1 >>>
PURPOSE:     Retreive the latent heating profiles using 
             a TRMM 2A25 product
INPUT DATA:  2A25 version 5 swath data
OUTPUT DATA: grided 4-byte binary file for GrADS display
             and GrADS ctl file
COMPILE:     make -f glhret.make   
USAGE:       glhret {Input file name} {GrADS Output file name}\ 
                     [start scan num] [end scan num]
             (it is possible to omit [] terms)
NOTE:        You need "param_lhret.dat" file on your 
             executing directory

***************************************************************/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "IO.h"             /* TSDIS Tool Kit */
#include "IO_PR.h"          /* TSDIS Tool Kit */
#include "IO_INTR_PR.h"     /* TSDIS Tool Kit */
#include "glhret_v3.0.h"    /* defines function prototypes */

#define  YDIM      49      /* number of angle bins in output */
#define  ZDIM      80      /* number of vertical bins in output */
#define  UNDEF   -999.9    /* missing data in output */
#define  ZRESO      0.25   /* vertical resolution in km */
#define  N3         3      /* number of raintype */
#define  N5         5      /* number of nodes for LWC-Z relations */
#define  N6         6      /* number of range bin numbers */
#define  N21       21      /* number of initial values for rho & vratio */
#define  NANGLE    49      /* number of angle bins */
#define  NRANGE    80      /* number of vertical bins */
#define  NFNAME   100      /* number of characters for a file name */
#define  PARAM_FILE "param_lhret.dat"   /* name of the parameter file */


int main(int argc, char *argv[])
{
/* ===========================================================================
      DEFINE VARIABLES
   =========================================================================== */
/* ------- variables of HDF/Toolkit ------- */
    L2A_25_SWATHDATA  read_data;
    IO_HANDLE         granuleHandle2A25;
    DATE_STR          beginDate;
    TIME_STR          beginTime;
    DATE_STR          endDate;
    TIME_STR          endTime;
    char  granuleID[NFNAME],algorithmVersion[NFNAME];
    char  filemode;
    int   orbitNumber,attitudeMode,productVersion;
    int   dataType,status,orbitSize;

/* ------- Common variables ------- */
    int   sscan,escan;
    int   i,ii,j,k,kk,k1,k2,l,lp1,irt;
    int   ihr,imin,isec;
    float stime,slat,slon;
    float first_scan_sec=0.0;

/* ------- Air parameters ------- */    
    float press_km[N21];               /* pressure at every km */
    float tempa_km[N21];               /* temparature at every km */
    float rho_km[N21];                 /* air dinsity at every km */
    float vratio_km[N21];              /* relative terminal velocities at every km */
    float press[NRANGE];               /* pressure (US standard) */
    float tempa[NRANGE];               /* temparature (US standard) */
    float qvs[NRANGE];                 /* saturated mixing ratio of vapor */
    float rho[NRANGE];                 /* air density (US standard) */
    float vratio[NRANGE];              /* relative terminal velocities  */

/* ------- WC-Ze relation --------- */
    char  init_file[NFNAME];           /* names of the parameter files */
    float zwc_a_c0[N3][N5];            /* LWC-Z coefficient, initial value */
    float zwc_a_c1[N3][N5];            /* a = 10^(a_c0+a_c1*epsilon+a_c2*epsilon^2) */
    float zwc_a_c2[N3][N5];            /*   where x = log10(epsilon) */
    float zwc_b_c0[N3][N5];            /* LWC-Z power, initial value */
    float zwc_b_c1[N3][N5];            /* b = 10^(b_c0+b_c1*epsilon+b_c2*epsilon^2) */
    float zwc_b_c2[N3][N5];            /*   where x = log10(epsilon) */
    float zwc_a[NRANGE];               /* coefficient in WC-Z relation */
    float zwc_b[NRANGE];               /* power in WC-Z relation */
    int   raintype;                    /* rain type (0=STRAT, 1=CONV, 2=OTHERS) */ 
    int   flag_warm;                   /* warm rain flag */

/* ------- 2A25 original/derived variable ------- */ 
    int   sfcBin, bbBin, cldtBin;
    int   rbinNum[N6];                 /* range bin numbers of top[0], bottom[1], 
                                          surface[2], BB[3], PIZ_threshold[4], ZeMax[5] */
    int   rbfsTop, rbfsBottom, rbfsSurf, rbfsBB;
    int   rbinNode[N5];                /* node range bin numbers of ZR coefficients */
    float epsilonF;                    /* attenuation correction factor */
    float weightF;                     /* weighting factor in the calc of epsilon */
    float nubfCF;                      /* non-uniform beam filling correction factor */
    double xze, xz, xrain, xwc, xqp, xvt, xvt0;
    static float  cze[ZDIM][YDIM];
    static float  rain[ZDIM][YDIM];
    static float  wc[ZDIM][YDIM];
    static float  qp[ZDIM][YDIM], xxqp[ZDIM][YDIM];
    static float  vt[ZDIM][YDIM];
    static float  vt0[ZDIM][YDIM];
    static float  zra[ZDIM][YDIM], zrb[ZDIM][YDIM];
    static float  zwa[ZDIM][YDIM], zwb[ZDIM][YDIM];
    static float  flag_rain[YDIM];  
    static float  flag_PIA[YDIM];
    static float  flag_rtype[YDIM];  
    static float  flag_BB[YDIM];
    static float  flag_bottom[YDIM];
    static float  flag_wtype[YDIM];
    static float  sfcr[YDIM];
    static float  sfcz[YDIM];
    static float  sfcht[YDIM];         /* bright band height [km] */
    static float  bbht[YDIM];          /* surface height [km] */

/* ------- latent heating output ------- */ 
    int   iteration;
    static float  trop[YDIM];          /* troppause height [km] */
    static float  cldt[YDIM];          /* cloud top height [km] */
    static float  cldb[YDIM];          /* cloud bottom height [km] */
    static float  w[ZDIM][YDIM];       /* vertical wind profile [m/s] */
    static float  fqp[ZDIM][YDIM];     /* production rate of qp [kg/s] */
    static float  lh[ZDIM][YDIM];      /* latent heating profile [K/hr] */

/* ------- other output variables ------- */    
    FILE  *fpg;
    char  g_name[NFNAME];
    int   m, n, mc, nc;
    int   outsize1, outsize2;
    char   c_var[400], c_memo[128];
    int    nx, ny, nz, nt, tdim, iday, imonth, iyear;
    double xmin, ymin, zmin, tmin, dx, dy, dz, dt, no_data;


/* ===========================================================================
      OPEN FILES, READ PARAMETERS
   =========================================================================== */
/* ------- open input file (HDF) ------- */
    if(argc < 3){
       printf("Usage: %s {Input file name} {GrADS Output file name} [start scan num] [end scan num]\n",argv[0]);
       exit(0);
    }

    sscan = 1;                              /* start scan num (default) */
    if(argc >= 4) sscan = atoi(argv[3]);
    escan = 9999;                           /* end scan num (default) */
    if(argc >= 5) escan = atoi(argv[4]);
    if(escan < sscan) escan = sscan;

    strcpy(granuleID,argv[1]);	
    if(access(granuleID, 000) == -1){
        printf("Input file name error!\n");
        exit(1);
    }

    dataType = TK_L2A_25;     /* TK_L2A_25 is defined in IO.h */
    filemode = TK_READ_ONLY;

    status = TKopen(granuleID, dataType, filemode, &granuleHandle2A25);
    if(status  != TK_SUCCESS) {
	printf("Toolkit file open error!\n");
        exit(1);
    }

/* ------- check metadata  ------ */
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_BEGIN_DATE, &beginDate);
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_BEGIN_TIME, &beginTime);
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_END_DATE, &endDate);
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_END_TIME, &endTime);
       status = TKreadMetadataChar(&granuleHandle2A25,
                        TK_GRANULE_ID, granuleID);
       status = TKreadMetadataChar(&granuleHandle2A25,
                        TK_ALGORITHM_VERSION, algorithmVersion);
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_ATTITUDE_MODE, &attitudeMode);
       status = TKreadMetadataInt(&granuleHandle2A25,
                        TK_ORBIT_SIZE, &orbitSize);

       printf("# granule ID    = %s\n", granuleID );
       printf("# algorithm Version = %s\n", algorithmVersion );
       printf("# attitude Mode = %d\n", attitudeMode );
       printf("# begin time    = %4d/%2d/%2d, %2d:%2d:%2d \n",
                 beginDate.tkyear, beginDate.tkmonth, beginDate.tkday,
                 beginTime.tkhour, beginTime.tkminute, beginTime.tksecond );
       printf("# end time      = %4d/%2d/%2d, %2d:%2d:%2d \n",
                 endDate.tkyear, endDate.tkmonth, endDate.tkday,
                 endTime.tkhour, endTime.tkminute, endTime.tksecond );
       printf("# num of scan   = %d\n\n",  orbitSize );

       if(orbitSize < escan) escan=orbitSize;
       printf("# output scan num: %d - %d\n\n",sscan,escan);


/* ------- read parameter file ------ */
       strcpy(init_file, PARAM_FILE);
       status = read_param(init_file, zwc_a_c0, zwc_a_c1, zwc_a_c2, 
                           zwc_b_c0, zwc_b_c1, zwc_b_c2, rho_km, 
                           tempa_km, press_km, vratio_km);
       if(status > 1){
	  printf("parameter file read error: %d data\n", status);
          if(status == 99) printf("parameter file open error!\n");
          exit(1);
       } 

/* ------- get air pressure, temparature, qvs, density, and vratio ------- */
       status = air_param(press_km, tempa_km, rho_km, vratio_km, 
                          press, tempa, qvs, rho, vratio);

/* ------- open output file (GrADS) ------- */
       strcpy(g_name,argv[2]);
       if((fpg = fopen(g_name,"w" )) == NULL ){
          printf("Output file open error!: %s \n",g_name); 
          exit(1);   
       }

/* ===========================================================================
      READ ACTUAL DATA and DERIVE WC, qp, Vt
   =========================================================================== */
/* ------- skip scan ------- */
       for(i=1; i<sscan; i++){
         status = TKreadScan(&granuleHandle2A25,&read_data);
         if(status != TK_SUCCESS){
            printf("### Read Error at scan num=%d\n",i);
         }
       }

/* ------- read scan by scan ------- */
       for(i=sscan; i<=escan; i++){      /* scan loop */
         status = TKreadScan(&granuleHandle2A25,&read_data) ;
         if(status != TK_SUCCESS){
            printf("### Read Error at scan num=%d\n",i);
         }else{
	   ii=i-sscan;    /* ii means output scan number, from 0 to escan-sscan */

         for(j=0; j<NANGLE; j++){           /* angle bin loop */
            stime = (float)read_data.scanTime;
            slat  = (float)read_data.geolocation[j][0];
            slon  = (float)read_data.geolocation[j][1];

            /*--- get surface Ze & rain ---*/
            sfcz[j] = (float)read_data.nearSurfZ[j];
            sfcr[j] = (float)read_data.nearSurfRain[j];

            /*--- get range bin number of the 5 Node ---*/
            for(l=0; l<N5; l++){
               rbinNode[l] = (int)read_data.ZRParmNode[j][l];
            }

            /*--- get surface & BB height [km] --------------------------
               range bin numbers of top[0], bottom[1],surface[2], BB[3],
                                    PIZ_threshold[4], ZeMax[5]
	    -------------------------------------------------------------*/
            for(l=0; l<N6; l++){
               rbinNum[l] = (int)read_data.rangeBinNum[j][l];
            }
            /* cldt[j]=(79-rbinNum[0])*ZRESO;    /* 1 km above the rain-certain bin */
            /* cldb[j]=(79-rbinNum[1])*ZRESO;    /* bottom range bin (+1) height */
            /* ---> cldt & cldb are defined by original cld_ht routine */
            sfcht[j] = (79-rbinNum[2])*ZRESO;    /* actual surface height */
            bbht[j]  = (79-rbinNum[3])*ZRESO;    /* bright band height */

	   
            /*--- rainfall flag in 2A25 ------------------------------
	          0: (bit  1) no rain
	   	 +1: (bit  1) rain possible
	   	 +2: (bit  2) rain certain
	   	 +4: (bit  3) zeta^beta > 0.5 (PIA larger than 3 dB)
	   	 +8: (bit  4) large attenuation (PIA larger than 10 dB)
	        +16: (bit  5) stratiform
	        +32: (bit  6) convective
	        +64: (bit  7) BB exist
	       +128: (bit  8) warm rain
	       +256: (bit  9) rain bottom above 2 km
	       +512: (bit 10) rain bottom above 4 km
	     +16384: (bit 15) data missing between rain top and bottom
	    -------------------------------------------------------------*/
            /*--- output flag as 2A25 original --------------------------
               flag_rain[49]:    0.=no_rain, 1.=possible, 2.=certain
               flag_PIA[49]:     0.<=3 db, 1.>3 db, 2.>10 db
	       flag_rtype[49]:   0.=strat, 1.=conv, 2.=others <<original>>
               flag_BB[49]:      0.=?, 1.=BB exist, 2.=warm_rain
               flag_rbottom[49]: 0.=normal, 1.>2 km, 2.>4 km, 3.=missing
 	    -------------------------------------------------------------*/
            flag_rain[j]=0.0;  flag_PIA[j]=0.0;  flag_rtype[j]=2.0;  
            flag_BB[j]=0.0;   flag_bottom[j]=0.0;
            if( (read_data.rainFlag[j]&1)      ==1)    flag_rain[j]=1.0;
            if(((read_data.rainFlag[j]&2)>>1)  ==1)    flag_rain[j]=2.0;
            if(((read_data.rainFlag[j]&4)>>2)  ==1)    flag_PIA[j]=1.0;
            if(((read_data.rainFlag[j]&8)>>3)  ==1)    flag_PIA[j]=2.0;
            if(((read_data.rainFlag[j]&16)>>4) ==1)    flag_rtype[j]=0.0;
            if(((read_data.rainFlag[j]&32)>>5) ==1)    flag_rtype[j]=1.0;
            if(((read_data.rainFlag[j]&64)>>6) ==1)    flag_BB[j]=1.0;
            if(((read_data.rainFlag[j]&128)>>7)==1)    flag_BB[j]=2.0;
            if(((read_data.rainFlag[j]&256)>>8)==1)    flag_bottom[j]=1.0;
            if(((read_data.rainFlag[j]&512)>>9)==1)    flag_bottom[j]=2.0;
            if(((read_data.rainFlag[j]&16384)>>14)==1) flag_bottom[j]=3.0;

            /*--- new flag (flag_wtype) to choice the w-profile shape ----
                flag_wtype 
                0.=NO_RAIN, 1.=CONV, 2.=STRAT, 3.=ANVIL, 4.=SHALLOW
 	    -------------------------------------------------------------*/
            rbfsTop    = 79-rbinNum[0];   /* Top range bin from surface */
            rbfsBottom = 79-rbinNum[1];   /* Bottom range bin from surface */
            rbfsSurf   = 79-rbinNum[2];   /* Surf range bin from surface */
            rbfsBB     = 79-rbinNum[3];   /* BB range bin from surface */
            /* if(flag_rain[j] > 0.9){                         /* rain possible */
            if(flag_rain[j] > 1.9){                         /* rain certain */
               if(flag_rtype[j]<0.1){                       /* original strat */
                  if(rbfsTop>=rbfsBB) flag_wtype[j]=2.0;  
                  if(rbfsTop<rbfsBB)  flag_wtype[j]=4.0;
               }
               if(0.9<flag_rtype[j] && flag_rtype[j]<1.1){  /* original conv */
		  flag_wtype[j]=1.0;
               }
               if(1.9<flag_rtype[j] && flag_rtype[j]<2.1){  /* original others */
                  flag_wtype[j]=3.0;
                  if(rbfsTop<rbfsBB)  flag_wtype[j]=4.0;
                  /* if(rbfsBottom>rbfsSurf) flag_wtype[j]=3.0; --- typical, but no need */
               }
            }else{
	       flag_wtype[j]=0.0;  /* no_rain */
            }


            /*--- raintype (-1=NO_RAIN, 0=STRAT, 1=CONV, 2=OTHERS) ------
                                                      for Z-WC relation 
                in this routine, rain possible & others => STRAT (anvil)
            -------------------------------------------------------------*/
            raintype=-1;
            if( (read_data.rainFlag[j]&1)      ==1) raintype=0; 
            if(((read_data.rainFlag[j]&16)>>4) ==1) raintype=0;
            if(((read_data.rainFlag[j]&32)>>5) ==1) raintype=1;
            if(((read_data.rainFlag[j]&128)>>7)==1) raintype=2;

            /*--- flag_warm (1=warm rain) for Z-WC relation ---*/
            flag_warm =0;
            if(((read_data.rainFlag[j]&128)>>7)==1) flag_warm=1;

            /*--- get coefficients of Z-WC relation ---*/
            nubfCF   = read_data.nubfCorrectFactor[j][1];
	    epsilonF = read_data.epsilon[j];
	    weightF  = read_data.weightW[j];
            /*-------------------------------------------------------- 
              If you use 2A25 version 4 product, remove the commentted 
              following 2 lines to use the fixed epsilon.  
                  epsilonF = 1.0;
                  weightF = 0.0;
	      ---------------------------------------------------------*/ 

            status = wc_param(rbinNum, raintype, flag_warm, zwc_a_c0, zwc_a_c1, zwc_a_c2, 
                              zwc_b_c0, zwc_b_c1, zwc_b_c2, nubfCF, epsilonF, 
                              weightF, rbinNode, zwc_a, zwc_b);

#ifdef DEBUG
            if(j==24){
               printf("# ii=%4d  nubfCF=%6.4f  epsilonF=%6.4f  weightF=%6.4f",
		      ii, nubfCF, epsilonF, weightF);
               printf("  rbinNode[0]=%2d  [1]=%2d  [2]=%2d  [3]=%2d  [4]=%2d\n",
		      rbinNode[0],rbinNode[1],rbinNode[2],rbinNode[3],rbinNode[4]);
               printf("                                                  ");
               printf("  rbinNum[0]=%2d  [1]=%2d  [2]=%2d  [3]=%2d  [4]=%2d  [5]=%2d\n",
		      rbinNum[0],rbinNum[1],rbinNum[2],rbinNum[3],rbinNum[4],rbinNum[5]);
            }
            if(j==24 && raintype!=-1){ 
               for(k=rbinNum[0]; k<rbinNum[1]; k++){
	         printf("# j=%2d  k=%2d  zwc_a=%10.4f  zwc_b=%10.4f\n",j,k,zwc_a[k],zwc_b[k]);
               }
	    }
#endif

            /*--- get Ze, Rain, WC, Vt ---------------------------------------*/
	    /* k is downward, which includes read_data.* (2A25), zwc_a, zwc_b */
	    /* kk is upward, which includes rho, vratio, and output variables */
            /* xrain and xze data includes missing value (-99.9) and          */
	    /*                             clutter value (-88.9)              */
            /*----------------------------------------------------------------*/
            for(k=0; k<NRANGE; k++){
               kk=NRANGE-1-k;                  /* the same as 79-k */
               xrain = (float)read_data.rain[j][k];             /* [mm/hr] */
               if(xrain < 0.0) xrain = 0.0;
               xze  = (float)read_data.correctZFactor[j][k];    /* [dBZ] */
               if(xze < 0.0) xze = 0.0;
               xz = pow(10.0,xze/10.0);                         /* [mm^6/m^3] */
               xwc = zwc_a[k] * pow(xz,zwc_b[k]);               /* [g/m^3] */
               xqp = xwc/rho[kk];                               /* [g/kg] */
               if(xwc > 0.0){
                  xvt = xrain/(3.6*xwc);                        /* [m/s] */
               }else{
		  xvt = 0.0;
               }
               xvt0 = xvt/vratio[kk];                           /* [m/s] */
               if(xrain > 0.0){
                  cze[kk][j]  = xze;
                  rain[kk][j] = xrain;
                  wc[kk][j] = xwc * 1.0e-3;       /* into [kg/m^3] */
                  qp[kk][j] = xqp * 1.0e-3;       /* into [kg/kg] */
                  vt[kk][j] = xvt * (-1.0);       /* upward is plus */
                  vt0[kk][j] = xvt0 * (-1.0);     /* upward is plus */
               }else{
                  cze[kk][j]  = UNDEF;
                  rain[kk][j] = UNDEF;
                  wc[kk][j] = UNDEF;
                  qp[kk][j] = UNDEF;
                  vt[kk][j] = UNDEF;
                  vt0[kk][j] = UNDEF;
               }
	    }


           if(first_scan_sec==0.0) first_scan_sec = stime;
           /* if(j==24 && (i==sscan || i==escan)){       /* on nadir */
           if(j==24){       /* on nadir */
             ihr =(int)(stime/3600.);
             imin=(int)((stime-ihr*3600.)/60.);
             isec=(int)(stime-ihr*3600.-imin*60.);
             printf("Scan:%4d  %2d:%2d:%2d  %7.3f,%8.3f \n",
                     i,ihr,imin,isec,slat,slon);
           }

         }  /* end of angle loop */
         }  /* if-else of readScan error */

/* ===========================================================================
      ESTIMATE w-, Fqp-, LH-profile
   =========================================================================== */
         /*--- get cloud top and bottom height ---*/
         status = cld_ht(cze, flag_rain, flag_rtype, rbinNum,
                         cldt, cldb, trop);

         /*--- smoothing ---*/
         for(j=0; j<YDIM; j++){
           for(k=0; k<ZDIM; k++){
             xxqp[k][j]=qp[k][j];
             qp[k][j]=0.0;
           }
           for(k=1; k<ZDIM-1; k++){
             qp[k][j]=(xxqp[k-1][j]+xxqp[k][j]+xxqp[k+1][j])/3.0;
           }
         }
         for(j=0; j<YDIM; j++){
           for(k=0; k<ZDIM; k++){
             xxqp[k][j]=qp[k][j];
             qp[k][j]=0.0;
           }
           for(k=1; k<ZDIM-1; k++){
             qp[k][j]=(xxqp[k-1][j]+xxqp[k][j]+xxqp[k+1][j])/3.0;
           }
         }

         /*--- retrieve LH profile ---*/
         iteration = lh_prof(rho, qvs, flag_wtype, sfcr,    
			     sfcht, bbht, cldt, cldb, trop, 
                             qp, vt, vt0, w, fqp, lh);


/* ===========================================================================
      OUTPUT DATA
   =========================================================================== */
          /*--- output GrADS file ---*/
          outsize1 = YDIM * sizeof(float);
          outsize2 = YDIM * ZDIM * sizeof(float);
          fwrite(cze,1,outsize2,fpg);
          fwrite(rain,1,outsize2,fpg);
          fwrite(wc,1,outsize2,fpg);
          fwrite(qp,1,outsize2,fpg);
          fwrite(vt,1,outsize2,fpg);
          fwrite(vt0,1,outsize2,fpg);
          fwrite(w,1,outsize2,fpg);
          fwrite(fqp,1,outsize2,fpg);
          fwrite(lh,1,outsize2,fpg);
          fwrite(sfcz,1,outsize1,fpg);
          fwrite(sfcr,1,outsize1,fpg);
          fwrite(sfcht,1,outsize1,fpg);
          fwrite(bbht,1,outsize1,fpg);
          fwrite(cldt,1,outsize1,fpg);
          fwrite(cldb,1,outsize1,fpg);
          fwrite(flag_rain,1,outsize1,fpg);
          fwrite(flag_PIA,1,outsize1,fpg);
          fwrite(flag_rtype,1,outsize1,fpg);
          fwrite(flag_BB,1,outsize1,fpg);
          fwrite(flag_bottom,1,outsize1,fpg);
          fwrite(flag_wtype,1,outsize1,fpg);

       }    /* end of scan loop */



       /*--- close the output file ---*/
       fclose(fpg);
       tdim = escan-sscan;
       printf("\n");
       printf("output data: %d(angle) * %d(range) * %d(scan) * 4 bytes\n",
               YDIM,ZDIM,tdim);
       printf("             3D(9) :ze,rain,wc,qp,vt,vt0,w,fqp,lh\n");
       printf("             2D(12):sfcz,sfcr,sfcht,bbht,cldt,cldb,flag(6)\n\n");

        /*--- output GrADS ctl file ---*/
        strcpy(c_memo,"2A25 Latent Heating Profile");
        strcpy(c_var,"ze,rain,wc,qp,vt,vt0,w,fqp,lh,sfcz,sfcr,sfcht,bbht,cldt,cldb,f_rain,f_PIA,f_rtype,f_BB,f_bottom,f_wtype");
        nx=1;          ny=YDIM;     nz=ZDIM;    nt=tdim;
        xmin=0.0;      ymin=0.0;    zmin=0.0;   tmin=0.0;
        dx=4.2;        dy=4.2;      dz=ZRESO;   dt=1.0;
        no_data= UNDEF;
        iyear  = beginDate.tkyear; 
        imonth = beginDate.tkmonth;
        iday   = beginDate.tkday;
        ihr    =(int)(first_scan_sec/3600);
        imin   =(int)((first_scan_sec-ihr*3600)/60);

        ggrads_ctl( g_name, nx, ny, nz, nt, xmin, ymin, zmin, tmin,
                    dx, dy, dz, dt, no_data, ihr, imin, iday, imonth, iyear,
                    c_var, c_memo );

        /*--- close HDF file ---*/
	status = TKclose(&granuleHandle2A25);


	return(0);
}
